#ifndef BPMBITMAP_H_
#define BPMBITMAP_H_

#include "IRect.h"
#include <String.h>
#include "TempBuffer.h"
#include "TileManager.h"

typedef enum
{
BPM_GRAYSCALE_BITMAP,
BPM_INDEXED_BITMAP,
BPM_RGB_BITMAP,
BPM_RGBA_BITMAP,
BPM_CMYK_BITMAP
} BPMBitmapType;

class BPMBitmap
{
public:
	BPMBitmap(const char *dname,const IRect &bounds,BPMBitmapType bmptype);
	~BPMBitmap(void);
	void MergeShadow(int16);
	void Fill(uint8 r,uint8 g, uint8 b, uint a);
	IRect MaskBounds(void);
	void InvalidatePreview(void);
	bool IsDirty(void);
	bool HasAlpha(void);
	BPMBitmapType Type(void);
	TileManager *GetTileManager(void);
	TileManager *GetShadowManager(void);
	int16 GetBytesPerPixel(void);
	IRect Bounds(void);
	bool IsVisible(void);
	IPoint	Offsets(void);
	const char *GetName(void);
	void SetName(const char *name);
	uint8 *GetColor(const IPoint &pt);
	int16 GetID(void) const { return id; }
	int16 GetImageID(void) const { return imageid; }
//	void SetImage(BPMImage *image);
//	BPMImage *GetImage(void) const;
//	unsigned char *GetColorMap(void);
//	void Deallocate(void);

	BString name;				// Name (title) of the bitmap
	TileManager *tiles;			// The real bitmap data
	TileManager *shadow;		
	bool visible;				// Is the bitmap visible?
	IRect frame;				// Size of the bitmap
	int16 offset_x, offset_y;	// Offset of bitmap when drawn in an image
	int16 bpp;					// Size in bytes per pixel. Either 1 or 4.
	bool dirty;					
	int16 id;					// Unique ID across the entire application
	int16 imageid;				// ID of owning image
	BPMBitmapType type;			// Color space of the bitmap
	
	TempBuffer *preview;		// Quick preview of the bitmap
	bool preview_valid;			// Does the preview reflect the current bitmap data?
};

#endif