#ifndef BPMIMAGE_H
#define BPMIMAGE_H

#include <SupportDefs.h>
#include <String.h>
#include "IRect.h"

class BPMImage
{
public:
	BString filename;
	bool filename_valid;
	IRect frame;
	BPMBitmapType type;		// Color space of the image. See BPMBitmap.h
	double xres,yres;		// resolution in DPI
	
	uint8 *palette;			// palette of colors used in indexed mode
	uint8 palette_used;		// number of color "slots" used in the palette
	bool dirty;				
	
	uint8 instance_count;	
	uint8 display_count;	
	
	TileManager *shadow;	// shadow tiles
	
	
	// Projection members.
	// A projection is a "flattened" image used to actually display the image on the
	// screen which consists of all Layer types of drawables. Masks are then applied
	// in order for it to be displayed properly, like which channels to display or
	// the current selection
	bool updating_projection;
	BPMBitmapType projection_type;	// color space of the projection
	uint8 projection_bpp;			// number of bytes per pixel for projection. 1 or 4
	uint8 projection_level;
	TileManager   *projection;		// Actual data for the projection
	

	BList *layers;			// List of all layers in the image
	BList *channels;		// List of channel masks
	BList *layer_stack;		// List which tracks the order of the layers
	
	Layer *active_layer;			// The current layer in use
	Channel *active_channel;		// Active channel mask
	Layer   *floating_selection;	// Active selection mask
	Channel *selection_mask;		
	
/*	ParasiteList *parasites;		//  Plug-in parasite data
	
	PathList *paths;				//  Paths data for this image
	
	int visible [MAX_CHANNELS];		//  visible channels
	int active  [MAX_CHANNELS];		//  active channels
*/

	//  Undo apparatus
/*	GSList *undo_stack;                 //  stack for undo operations
	GSList *redo_stack;                 //  stack for redo operations
	gint    undo_bytes;                 //  bytes in undo stack
	gint    undo_levels;                //  levels in undo stack
	gint    group_count;		      //  nested undo groups
	UndoType pushing_undo_group;        //  undo group status flag
	GtkWidget *undo_history;	      //  history viewer, or NULL
*/	

	TempBuffer *composite_preview;	// preview of the projection
	bool composite_preview_valid;	// Is the projection preview valid?
};

#endif