//------------------------------------------------------------------------------
//	Copyright (c) 2001-2002, OpenBeOS
//
//	Permission is hereby granted, free of charge, to any person obtaining a
//	copy of this software and associated documentation files (the "Software"),
//	to deal in the Software without restriction, including without limitation
//	the rights to use, copy, modify, merge, publish, distribute, sublicense,
//	and/or sell copies of the Software, and to permit persons to whom the
//	Software is furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//	DEALINGS IN THE SOFTWARE.
//------------------------------------------------------------------------------
#ifndef	_IPOINT_H
#define	_IPOINT_H

#include <SupportDefs.h>

class IRect;

// IPoint class ----------------------------------------------------------------
class IPoint {
public:
	int32 x;
	int32 y;

	IPoint();
	IPoint(int32 X, int32 Y);
	IPoint(const IPoint &p);
		
	IPoint	&operator=(const IPoint &p);
	void	Set(int32 X, int32 Y);

	void	ConstrainTo(IRect r);
	void	PrintToStream() const;
			
	IPoint	operator+(const IPoint &p) const;
	IPoint	operator-(const IPoint &p) const;
	IPoint&	operator+=(const IPoint &p);
	IPoint&	operator-=(const IPoint &p);

	bool	operator!=(const IPoint &p) const;
	bool	operator==(const IPoint &p) const;
};
//------------------------------------------------------------------------------

extern _IMPEXP_BE const IPoint B_ORIGIN;	// returns (0,0)

//------------------------------------------------------------------------------
inline IPoint::IPoint()
{
	x = y = 0;
}
//------------------------------------------------------------------------------
inline IPoint::IPoint(int32 X, int32 Y)
{
	x = X;
	y = Y;
}
//------------------------------------------------------------------------------
inline IPoint::IPoint(const IPoint& pt)
{
	x = pt.x;
	y = pt.y;
}
//------------------------------------------------------------------------------
inline IPoint &IPoint::operator=(const IPoint& from)
{
	x = from.x;
	y = from.y;
	return *this;
}
//------------------------------------------------------------------------------
inline void IPoint::Set(int32 X, int32 Y)
{
	x = X;
	y = Y;
}
//------------------------------------------------------------------------------

#endif
