//------------------------------------------------------------------------------
//	Copyright (c) 2001-2002, OpenBeOS
//
//	Permission is hereby granted, free of charge, to any person obtaining a
//	copy of this software and associated documentation files (the "Software"),
//	to deal in the Software without restriction, including without limitation
//	the rights to use, copy, modify, merge, publish, distribute, sublicense,
//	and/or sell copies of the Software, and to permit persons to whom the
//	Software is furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//	DEALINGS IN THE SOFTWARE.
//------------------------------------------------------------------------------

#include <stdio.h>
#include "IRect.h"

bool TestLineIntersect(const IRect& r, int32 x1, int32 y1, int32 x2, int32 y2,
					   bool vertical = true);

//------------------------------------------------------------------------------
void IRect::InsetBy(IPoint p)
{
	 left += p.x;
	 right -= p.x;
	 top += p.y;
	 bottom -= p.y;
}
//------------------------------------------------------------------------------
void IRect::InsetBy(int32 dx, int32 dy)
{
	 left += dx;
	 right -= dx;
	 top += dy;
	 bottom -= dy;
}
//------------------------------------------------------------------------------
IRect& IRect::InsetBySelf(IPoint p)
{
	this->InsetBy(p);
	return *this;
}
//------------------------------------------------------------------------------
IRect& IRect::InsetBySelf(int32 dx, int32 dy)
{
	this->InsetBy(dx, dy);
	return *this;
}
//------------------------------------------------------------------------------
IRect IRect::InsetByCopy(IPoint p)
{
	IRect copy(*this);
	copy.InsetBy(p);
	return copy;
}
//------------------------------------------------------------------------------
IRect IRect::InsetByCopy(int32 dx, int32 dy)
{
	IRect copy(*this);
	copy.InsetBy(dx, dy);
	return copy;
}
//------------------------------------------------------------------------------
void IRect::OffsetBy(IPoint p)
{
	 left += p.x;
	 right += p.x;
	 top += p.y;
	 bottom += p.y;
}
//------------------------------------------------------------------------------
void IRect::OffsetBy(int32 dx, int32 dy)
{
	 left += dx;
	 right += dx;
	 top += dy;
	 bottom += dy;
}
//------------------------------------------------------------------------------
IRect& IRect::OffsetBySelf(IPoint p)
{
	this->OffsetBy(p);
	return *this;
}
//------------------------------------------------------------------------------
IRect& IRect::OffsetBySelf(int32 dx, int32 dy)
{
	this->OffsetBy(dx, dy);
	return *this;
}
//------------------------------------------------------------------------------
IRect IRect::OffsetByCopy(IPoint p)
{
	IRect copy(*this);
	copy.OffsetBy(p);
	return copy;
}
//------------------------------------------------------------------------------
IRect IRect::OffsetByCopy(int32 dx, int32 dy)
{
	IRect copy(*this);
	copy.OffsetBy(dx, dy);
	return copy;
}
//------------------------------------------------------------------------------
void IRect::OffsetTo(IPoint p)
{
	 right = (right - left) + p.x;
	 left = p.x;
	 bottom = (bottom - top) + p.y;
	 top = p.y;
}
//------------------------------------------------------------------------------
void IRect::OffsetTo(int32 x, int32 y)
{
	 right = (right - left) + x;
	 left = x;
	 bottom = (bottom - top) + y;
	 top=y;
}
//------------------------------------------------------------------------------
IRect& IRect::OffsetToSelf(IPoint p)
{
	this->OffsetTo(p);
	return *this;
}
//------------------------------------------------------------------------------
IRect& IRect::OffsetToSelf(int32 dx, int32 dy)
{
	this->OffsetTo(dx, dy);
	return *this;
}
//------------------------------------------------------------------------------
IRect IRect::OffsetToCopy(IPoint p)
{
	IRect copy(*this);
	copy.OffsetTo(p);
	return copy;
}
//------------------------------------------------------------------------------
IRect IRect::OffsetToCopy(int32 dx, int32 dy)
{
	IRect copy(*this);
	copy.OffsetTo(dx, dy);
	return copy;
}
//------------------------------------------------------------------------------
void IRect::PrintToStream() const
{
	printf("(l:%ld t:%ld r:%ld b:%ld)\n", left, top, right, bottom);
}
//------------------------------------------------------------------------------
bool IRect::operator==(IRect r) const
{
	 return left == r.left && right == r.right &&
	 		top == r.top && bottom == r.bottom;
}
//------------------------------------------------------------------------------
bool IRect::operator!=(IRect r) const
{
	 return !(*this == r);
}
//------------------------------------------------------------------------------
IRect IRect::operator&(IRect r) const
{
	 return IRect(max_c(left, r.left), max_c(top, r.top),
	 			  min_c(right, r.right), min_c(bottom, r.bottom));
}
//------------------------------------------------------------------------------
IRect IRect::operator|(IRect r) const
{
	 return IRect(min_c(left, r.left), min_c(top, r.top),
	 			  max_c(right, r.right), max_c(bottom, r.bottom));
}
//------------------------------------------------------------------------------
bool IRect::Intersects(IRect r) const
{
	return	TestLineIntersect(*this, r.left, r.top, r.left, r.bottom)		||
			TestLineIntersect(*this, r.left, r.top, r.right, r.top, false)	||
			TestLineIntersect(*this, r.right, r.top, r.right, r.bottom)		||
			TestLineIntersect(*this, r.left, r.bottom, r.right, r.bottom, false);
}
//------------------------------------------------------------------------------
bool IRect::Contains(IPoint p) const
{
	return p.x >= left && p.x <= right && p.y >= top && p.y <= bottom;
}
//------------------------------------------------------------------------------
bool IRect::Contains(IRect r) const
{
	return r.left >= left && r.right <= right &&
		   r.top >= top && r.bottom <= bottom;
}
//------------------------------------------------------------------------------


//------------------------------------------------------------------------------
bool TestLineIntersect(const IRect& r, int32 x1, int32 y1, int32 x2, int32 y2,
					   bool vertical)
{
	if (vertical)
	{
		return	(x1 >= r.left && x1 <= r.right) &&
				((y1 >= r.top && y1 <= r.bottom) ||
				 (y2 >= r.top && y2 <= r.bottom));
	}
	else
	{
		return	(y1 >= r.top && y1 <= r.bottom) &&
				((x1 >= r.left && x1 <= r.right) ||
				 (x2 >= r.left && x2 <= r.right));
	}
}
