//------------------------------------------------------------------------------
//	Copyright (c) 2001-2002, OpenBeOS
//
//	Permission is hereby granted, free of charge, to any person obtaining a
//	copy of this software and associated documentation files (the "Software"),
//	to deal in the Software without restriction, including without limitation
//	the rights to use, copy, modify, merge, publish, distribute, sublicense,
//	and/or sell copies of the Software, and to permit persons to whom the
//	Software is furnished to do so, subject to the following conditions:
//
//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.
//
//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//	DEALINGS IN THE SOFTWARE.
//------------------------------------------------------------------------------
#ifndef	_IRECT_H
#define	_IRECT_H

#include <SupportDefs.h>
#include "IPoint.h"

class IRect {
public:
	int32	left;
	int32	top;
	int32	right;
	int32	bottom;

	IRect();
	IRect(const IRect &r);
	IRect(int32 l, int32 t, int32 r, int32 b);
	IRect(IPoint lt, IPoint rb);

	IRect	&operator=(const IRect &r);
	void	Set(int32 l, int32 t, int32 r, int32 b);

	void	PrintToStream() const;

	IPoint	LeftTop() const;
	IPoint	RightBottom() const;
	IPoint	LeftBottom() const;
	IPoint	RightTop() const;

	void	SetLeftTop(const IPoint p);
	void	SetRightBottom(const IPoint p);
	void	SetLeftBottom(const IPoint p);
	void	SetRightTop(const IPoint p);

	// transformation
	void	InsetBy(IPoint p);
	void	InsetBy(int32 dx, int32 dy);
	void	OffsetBy(IPoint p);
	void	OffsetBy(int32 dx, int32 dy);
	void	OffsetTo(IPoint p);
	void	OffsetTo(int32 x, int32 y);

	// expression transformations
	IRect &	InsetBySelf(IPoint);
	IRect &	InsetBySelf(int32 dx, int32 dy);
	IRect	InsetByCopy(IPoint);
	IRect	InsetByCopy(int32 dx, int32 dy);
	IRect &	OffsetBySelf(IPoint);
	IRect &	OffsetBySelf(int32 dx, int32 dy);
	IRect	OffsetByCopy(IPoint);
	IRect	OffsetByCopy(int32 dx, int32 dy);
	IRect &	OffsetToSelf(IPoint);
	IRect &	OffsetToSelf(int32 dx, int32 dy);
	IRect	OffsetToCopy(IPoint);
	IRect	OffsetToCopy(int32 dx, int32 dy);

	// comparison
	bool	operator==(IRect r) const;
	bool	operator!=(IRect r) const;

	// intersection and union
	IRect	operator&(IRect r) const;
	IRect	operator|(IRect r) const;

	bool	Intersects(IRect r) const;
	bool	IsValid() const;
	int32	Width() const;
	int32	Height() const;
	bool	Contains(IPoint p) const;
	bool	Contains(IRect r) const;

};
//------------------------------------------------------------------------------

// inline definitions ----------------------------------------------------------

inline IPoint IRect::LeftTop() const
{
	return(*((const IPoint*)&left));
}

inline IPoint IRect::RightBottom() const
{
	return(*((const IPoint*)&right));
}

inline IPoint IRect::LeftBottom() const
{
	return(IPoint(left, bottom));
}

inline IPoint IRect::RightTop() const
{
	return(IPoint(right, top));
}

inline IRect::IRect()
{
	top = left = 0;
	bottom = right = -1;
}

inline IRect::IRect(int32 l, int32 t, int32 r, int32 b)
{
	left = l;
	top = t;
	right = r;
	bottom = b;
}

inline IRect::IRect(const IRect &r)
{
	left = r.left;
	top = r.top;
	right = r.right;
	bottom = r.bottom;
}

inline IRect::IRect(IPoint leftTop, IPoint rightBottom)
{
	left = leftTop.x;
	top = leftTop.y;
	right = rightBottom.x;
	bottom = rightBottom.y;
}

inline IRect &IRect::operator=(const IRect& from)
{
	left = from.left;
	top = from.top;
	right = from.right;
	bottom = from.bottom;
	return *this;
}

inline void IRect::Set(int32 l, int32 t, int32 r, int32 b)
{
	left = l;
	top = t;
	right = r;
	bottom = b;
}

inline bool IRect::IsValid() const
{
	if (left <= right && top <= bottom)
		return true;
	else
		return false;
}

inline int32 IRect::Width() const
{
	return(right - left);
}

inline int32 IRect::Height() const
{
	return(bottom - top);
}

#endif
