#include <OS.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include "TempBuffer.h"

#ifdef DEBUG_TEMPBUFFER
#include <stdio.h>
#define STRACE(a) printf a
#else
#define STRACE(a) /* nothing */
#endif

/*
#ifdef HAVE_UNISTD_H
#endif
#include <string.h>
#include <sys/types.h>

#ifdef G_OS_WIN32
#include <process.h>		// For _getpid()
#endif
 
#include "apptypes.h"

#include "appenv.h"
#include "drawable.h"
#include "gdisplay.h"
#include "gimprc.h"
#include "paint_funcs.h"

#include "libgimp/gimpcolorspace.h"


static guchar * temp_buf_allocate (guint);
static void     temp_buf_to_color (TempBuf *, TempBuf *);
static void     temp_buf_to_gray  (TempBuf *, TempBuf *);
*/

// TODO: FIXME: Temporary hack. This should be based on the levels which the human eye
// perceives color, like red and blue aren't perceived as sensitively as green
#define INTENSITY(a,b,c) ( (a+b+c)/3 )

TempBuffer::TempBuffer(int16 w,int16 h,int16 bpp,int16 X,int16 Y,uint8 *col)
{	
	width=w;
	height=h;
	bytes=bpp;
	x=X;
	y=Y;
	swapped=false;
	
	data=new uint8[width * height * bytes];

	//  initialize the data, if possible
	if(col)
	{
		// First check if we can save a lot of work
		if (bytes==1)
		{
			memset (data, *col, width * height);
		}
		else
			if ((bytes == 3) && (col[1] == *col) && (*col == col[2]))
			{
				memset (data, *col, width * height * 3);
			}
			else
				if ((bytes==4) && (col[1]==*col) && (*col==col[2]) && (col[2]==col[3]))
				{
					memset (data, *col, (width * height) << 2);
				}
				else
				{
					// No, we cannot
					uint8 *index;
					
					// Fill the first row
					index=data;
					for (int32 i = width - 1; i >= 0; --i)
					{
						uint8 *init;
						int16 j=bytes;
						init = col;
						while (j--)
							*index++ = *init++;
					}
					
					// Now copy each row into successive ones
					bytes *= width;
					while (--height)
					{
						memcpy (index, data, bytes);
						index += bytes;
					}
				}
	}
	else
		memset (data, 0, width * height* bytes);
}

TempBuffer::~TempBuffer(void)
{
	if(data)
		delete data;

	if(swapped)
		SwapFree();
}

uint8 *TempBuffer::GetData(void)
{
	if(swapped)
		Unswap();
	
	return data;
}
void TempBuffer::CopyBuffer(TempBuffer *from)
{
	if(!from)
	{
		STRACE(("NULL source pointer in TempBuffer::CopyBuffer\n"));
		return;
	}
	
	if( (from->width!=width) || (from->height!=height) )
	{
		STRACE(("Inequal buffer dimensions in TempBuffer::CopyBuffer\n"));
		return;
	}
	
	if(bytes!=from->bytes)
	{
		// Convert to other color space, if possible.
		if(from->bytes==4)	// RGBA color -> grayscale
			CopyFromRGBA(from);
		else
		if(from->bytes==2)
			CopyFromGrayscale(from);
		else
		{
			STRACE(("Unable to convert from indexed color depth in TempBuffer::CopyBuffer\n"));
			return;
		}
		
	}
}

void TempBuffer::Swap(void)
{
//	TempBuf * swap;
//	gchar * filename;
	struct stat stat_buf;
	int16 err;
	FILE * fp;
	
	if (swapped)
		return;
	
	swapped=true;
	
/*	if(stingy_memory_use)
		swap=buf;
	else
	{
		swap = cached_in_memory;
		cached_in_memory = buf;
	}
*/
	
	//  For the case where there is no temp buf ready to be moved to disk, return
//	if (!swap)
//		return;
	
	// Get a unique filename for caching the data to a UNIX file
	char string[255];
	sprintf(string,"bephotomagic_%lu%d",real_time_clock(),rand());
	filename.SetTo(string);
	
	// Check if generated filename is valid
	err = stat(filename.String(), &stat_buf);
	if (!err)
	{
		if (stat_buf.st_mode & S_IFDIR)
		{
			STRACE(("Couldn't overwrite directory %s with a file in TmepBuffer::Swap()\n", filename.String()));
			filename.SetTo("");
			return;
		}
	}
	
	//  Open file for overwrite
	if((fp=fopen(filename.String(), "wb")))
	{
		size_t blocks_written;
		blocks_written = fwrite (data, width * height * bytes, 1, fp);

		// Check whether all bytes were written and fclose() was able to flush its buffers
		if ((fclose(fp)!=0) || (blocks_written!=1))
		{
			unlink(filename.String());
			STRACE(("Error writing to file %s in TempBuffer::Swap()\n",filename.String()));
			filename.SetTo("");
			return;
		}
	}
	else
	{
		unlink(filename.String());
		STRACE(("Error writing to file %s in TempBuffer::Swap()\n",filename.String()));
		filename.SetTo("");
		return;
	}
	delete data;
	data=NULL;
	
//	swap->filename = filename;
}

void TempBuffer::Unswap(void)
{
	struct stat stat_buf;
	FILE * fp;
	bool success=false;
	
	if (!swapped)
		return;
	
	swapped=false;
	
	//  If the requested temp buf is still in memory, simply return
/*	if (cached_in_memory == buf)
	{
		cached_in_memory = NULL;
		return;
	}
*/
	
	//  Allocate memory for the buffer's data
	data=new uint8[width*height*bytes];
	
	//  Find out if the filename of the swapped data is an existing file...
	if (!stat (filename.String(), &stat_buf))
	{
		if ((fp=fopen(filename.String(), "rb")))
		{
			size_t blocks_read;
			blocks_read=fread(data,width * height * bytes, 1, fp);
			(void) fclose (fp);
			if (blocks_read!=1)
				STRACE(("Error reading data in TempBuffer::Unswap()\n"));
			else
				success=true;
		}
		else
			STRACE(("Caching error in TempBuffer::Unswap()\n"));
	
		unlink(filename.String());
	}
	
	if(!success)
		STRACE(("Cached data somehow lost in TempBuffer::Unswap()!\n"));
	
	filename.SetTo("");
}

void TempBuffer::SwapFree(void)
{
	struct stat stat_buf;
	
	if (!swapped)
		return;
	
	//  Set the swapped flag
	swapped=false;
	
	// If the requested temp buf is cached in memory...
/*	if(cached_in_memory==buf)
	{
		cached_in_memory = NULL;
		return;
	}
*/	
	// Find out if the filename of the swapped data is an existing file...
	if (!stat(filename.String(), &stat_buf))
		unlink(filename.String());
	else
		STRACE(("Error in TempBuffer::SwapFree: information swapped to disk was lost!\n"));
	
	filename.SetTo("");
}

void TempBuffer::CopyFromGrayscale(TempBuffer *from)
{
	uint8 *src;
	uint8 *dest;
	long num_bytes;
	
	src=from->data;
	dest=data;
	
	if(!from)
	{
		STRACE(("NULL source pointer in TempBuffer::CopyFromGrayscale\n"));
		return;
	}
	
	num_bytes=from->width * from->height;
	
	while (num_bytes--)
	{
		uint8 tmpch;
		*dest++ = *src++;
		*dest++ = tmpch = *src++;
		*dest++ = tmpch;
		*dest++ = tmpch;
	}
}

void TempBuffer::CopyFromRGBA(TempBuffer *from)
{
	uint8 *src;
	uint8 *dest;
	long  num_bytes;
	float pix;
	
	src=from->data;
	dest=data;
	
	if(!from)
	{
		STRACE(("NULL source pointer in TempBuffer::CopyFromRGBA\n"));
		return;
	}

	num_bytes=from->width * from->height;
	
	while (num_bytes--)
	{
		*dest++ = src[0];
		
		pix=INTENSITY(src[1], src[2], src[3]);
		*dest++ = (uint8) pix;
		
		src += 4;
	}
}

MaskBuffer::MaskBuffer(int16 width, int16 height)
 : TempBuffer(width,height,1,0,0,NULL)
{
}

