/*
	Tile.cpp:	Tile class on which BPM's memory manager is built
*/
#include "Tile.h"

//#define DEBUG_TILE
//#define LOG_TILE

#ifdef DEBUG_TILE
#include <stdio.h>
#define STRACE(a) printf a

#ifdef LOG_TILE
#define SLOG(a) printf a
#else
#define SLOG(a) /* nothing */
#endif

#else
#define STRACE(a) /* nothing */

#ifdef LOG_TILE
#define SLOG(a) printf a
#else
#define SLOG(a) /* nothing */
#endif

#endif

uint32 global_tile_count=0;
uint32 active_tile_count=0;
uint32 shared_tile_count=0;

Tile::Tile(uint8 bytes_per_pixel)
{
	SLOG(("Tile::Tile(%d)\n",bytes_per_pixel));
	refcount=0;
	sharecount=0;
	width=MAX_TILE_WIDTH;
	height=MAX_TILE_HEIGHT;
	bpp=bytes_per_pixel;
	dirty=false;
	swapfile_id=-1;
	swap_offset=-1;
	swapped=false;
	data=NULL;
	global_tile_count++;
	links=NULL;
//	sem=-1;
}

Tile::~Tile(void)
{
	SLOG(("Tile::~Tile()\n"));

	if(refcount>0)
		debugger("Attempt to delete a referenced tile");

	if(sharecount>0)
		debugger("Attempt to delete a shared tile");

	FreeBuffer();
	
//	delete_sem(sem);
	
	global_tile_count--;
}

void Tile::Validate(void)
{
	AllocateBuffer();
//	sem=create_sem(1,"tile_sem");
}

void Tile::Invalidate(void)
{
	FreeBuffer();
//	delete_sem(sem);
}

void Tile::AllocateBuffer(void)
{
	SLOG(("Tile::AllocateBuffer\n"));
	if(data)
		return;
	
	data=new uint8[BitsLength()];
}

void Tile::FreeBuffer(void)
{
	SLOG(("Tile::FreeBuffer\n"));
	if(!data)
		return;
	
	if(refcount>0)
	{
		STRACE(("Attempted to free the buffer of a referenced tile\n"));
//		return;
	}
	if(sharecount>0)
	{
		STRACE(("Attempted to free the buffer of a shared tile\n"));
//		return;
	}
	delete data;
	data=NULL;
}

void Tile::Lock(void)
{
//	acquire_sem(sem);
}

void Tile::Unlock(void)
{
//	release_sem(sem);
}

bool Tile::IsLocked(void)
{
//	sem_info si;
//	get_sem_info(sem,&si);
//	return (si.count>0)?true:false;
	return false;
}

Tile &Tile::operator=(const Tile &from)
{
	dirty=from.dirty;
	width=from.width;
	height=from.height;
	bpp=from.bpp;

	if(from.data)
	{
		if(!data)
			AllocateBuffer();
		
		memcpy(data,from.data,from.BitsLength());
	}
	else
	{
		if(data)
			FreeBuffer();
	}
	return *this;
}
