#ifndef TILE_H_
#define TILE_H_

#include <OS.h>
#include "LinkList.h"

class TileManager;
class TileVMM;
class TileCache;
class TileLink;

#define MAX_TILE_WIDTH 64
#define MAX_TILE_HEIGHT 64
#define MAX_TILE_SIZE 4096

class TileLink
{
public:
	TileLink(void) { next=NULL; id=-1; manager=NULL; }
	~TileLink(void) {}
	TileLink *next;
	int16 id;
	void *manager;
};

class Tile : public LinkListItem
{
public:
	Tile(uint8 bpp);
	~Tile(void);
	void SetDirty(bool value) { dirty=value; }
	bool IsDirty(void) const { return dirty; }
	
	bool IsValid(void) const { return (data!=NULL); }
	uint8 GetWidth(void) const { return width; }
	uint8 GetHeight(void) const { return height; }
	uint8 BytesPerPixel(void) const { return bpp; }
	uint32 BitsLength(void) const { return (width * height * bpp); }


	void Lock(void);
	void Unlock(void);
	bool IsLocked(void);
	
	uint8 GetRefCount(void) const { return refcount; }
	uint8 *GetData(void) const { return data; }
	Tile &operator=(const Tile &from);
	int32 GetID(void) const { return id; }

private:
	friend TileCache;
	friend TileVMM;
	friend TileManager;

	void AllocateBuffer(void);
	void FreeBuffer(void);

	void Validate(void);
	void Invalidate(void);
	bool IsSwapped(void) const { return swapped; }
	off_t GetSwapOffset(void) const { return swap_offset; }
	void SetSwapOffset(off_t newoffset) { swap_offset=newoffset; }
	int16 GetSwapID(void) const { return swapfile_id; }
	void SetSwapID(int16 newid) { swapfile_id=newid; }
	
	bool dirty;				// Is the tile dirty?
	bool swapped;			// Is the tile swapped?
	uint8 refcount;			// how many times is the tile referenced
	uint8 sharecount;		// how many times is the tile referenced by a TileManager
	uint8 width, height;	// actual tile dimensions. May be less than TILE_HEIGHT/WIDTH
	uint8 bpp;				// # of bytes used for each pixel
	int32 id;				// # ID of a tile which is unique to a Tile VMM
	int16 swapfile_id;		// identifier of the swap file the tile resides in
	off_t swap_offset;		// file offset of the tile's data in the swapfile
	uint8 *data;			// the tile's image data
//	sem_id sem;				// semaphore for making things threadsafe
	TileLink *links;		// Links to all the TileManagers which reference this tile
};

#endif
