/*
	TileList.cpp:	Quick-and-dirty singly-linked list for tiles
*/
#include "TileList.h"
#include "Tile.h"

TileList::TileList(void)
{
	head=NULL;
	itemcount=0;
}

// This merely deletes the list pointers. It is the responsibility of the creator
// of the tiles to delete them
TileList::~TileList(void)
{
	TileListItem *item,*next;
	for(item=head; item!=NULL; item=next)
	{
		next=item->next;
		delete item;
	}
}

void TileList::Add(Tile *tile)
{
	if(!tile)
		return;
	
	TileListItem *item=new TileListItem(tile,head);
	head=item;
	itemcount++;
}

void TileList::Remove(int32 tileid)
{
	if(!head)
		return;
	
	// As is typical for a singly-linked list, we remove an item from the list merely
	// by making the item before it point to the item after it (or NULL if there isn't one).
	// There are 4 possible cases: item to be removed is the head item, the last item, or in the
	// middle.
	
	// 1) If the item is at the beginning, we just make the next item the head.
	// 2) Item is in the middle, change pointers as described above.
	// 3) Item is at the end, make the next-to-last item's pointer NULL
	// 4) Item is not in the list, do nothing.
	

	// Pointers to help track everything
	TileListItem *prev=NULL, *next=NULL, *toremove=NULL, *index;

	// Start at the beginning of the list and iterate through the whole thing
	
	if(head->data->GetID()!=tileid)
	{
		for(index=head; index!=NULL; index=index->next)
		{
			// See if the next item in the list is the one we're looking for. If it is, then we
			// want to save the current item as *prev (the one before the item to be removed) and
			// save the item after the one we want to remove as *next
			
			if(index->next && index->next->data->GetID()==tileid)
			{
				prev=index;
				next=index->next->next;
				toremove=index->next;
				break;
			}
		}

		if(toremove)
		{
			prev->next=next;
			delete toremove;
		}
	}
	else
	{
		// the head item is the one to remove. 
		toremove=head;
		head=toremove->next;
		delete toremove;
	}
	itemcount--;
}

Tile *TileList::Get(int32 tileid)
{
	TileListItem *index;
	for(index=head; index!=NULL; index=index->next)
	{
		if(index->data->GetID()==tileid)
			return index->data;
	}
	return NULL;
}

Tile *TileList::GetOldest(void)
{
	if(!head)
		return NULL;

	TileListItem *index;

	for(index=head; index!=NULL; index=index->next)
	{
		if(index->next==NULL)
			return index->data;
	}
	
	// shut up the compiler
	return NULL;
}
