#ifndef TILEMANAGER_H_
#define TILEMANAGER_H_

#include <SupportDefs.h>
#include "Tile.h"

#define TILE_READ_ONLY 1
#define TILE_READ_WRITE 3

class TileCache;
class TileVMM;

class TileManager
{
public:
	TileManager(int16 width, int16 height, int16 bytesperpixel);
	~TileManager(void);
	Tile *GetTile(int16 x, int16 y);
	Tile *GetTile(const int32 &tileid);
	int32 GetTileID(int16 x, int16 y);
	void MapTile(int16 x, int16 y, Tile *source);
	void MapTile(int32 tileid, Tile *source);
	void MapOverTile(Tile *source, Tile *dest);
	
	void ValidateTile(Tile *tile);
	void Invalidate(Tile **tileptr,int16 tileid);
	void Invalidate(Tile **tileptr,int16 x, int16 y);
	void InvalidateTiles(Tile *toplevel);
	void InvalidateTile(Tile *tile);
	void GetTileCoordinates(Tile *tile, int16 *x, int16 *y);

	void AttachTile(Tile *tile, int16 tileid);
	void DetachTile(Tile *tile, int16 tileid);
protected:	
	void AllocateTiles(int32 tilecount);
	
	int16 x,y;
	int16 width, height, bpp;
	int16 rowcount, colcount;
	Tile **tiles;

	TileVMM *vmm;
};

#endif