#ifndef TILEVMM_H_
#define TILEVMM_H_

#include <hash_map>
#include <String.h>
#include <OS.h>
#include <Locker.h>
#include "Tile.h"
#include "TileCache.h"
#include "TileList.h"

class SwapFile;


class TileVMM : public BLocker
{
public:
	TileVMM(const char *swapfilepath);
	~TileVMM(void);
	Tile *CreateTile(int16 bytesperpixel);
	void DestroyTile(int32 tileid);
	Tile *DuplicateTile(Tile *tile);
	Tile *GetTile(const int32 &tileid);
	void Validate(Tile *tile);
	void Invalidate(Tile *tile);
	uint64 CalculateNeededSpace(uint16 x, uint16 y, uint8 bpp);
protected:
	friend class TileCache;
	int16 CreateSwapFile(const char *filename);
	void DeleteSwapFile(int16 swapid);
	
	void SwapTileOut(Tile *tile);
	void SwapTileIn(Tile *tile);
	void RemoveTileFromSwap(Tile *tile);

	void ResizeFile(SwapFile *file, off_t newsize);
	uint64 ObtainOffset(SwapFile *file, uint16 size);
	
	TileCache *vmcache;
	
//	std::hash_map <int16, SwapFile *, hash<int> > swaplist;
	SwapFile *swapfile;
	TileList tilelist;

	BString swapfilepath;
};

#endif
