#include "ComboWindow.h"
#include "ComboBox.h"

#include <ListView.h>
#include <ScrollView.h>
#include <Debug.h>
#include <ClassInfo.h>

enum{
	M_LIST_INVOKE = 'mLII'
};

/**********************************************************
 * Constructor
 ***********************************************************/
ComboWindow::ComboWindow(BRect rect,int32 display,ComboBox* target)
		:BWindow(rect,NULL,
				B_BORDERED_WINDOW_LOOK,B_FLOATING_APP_WINDOW_FEEL,
				B_AVOID_FRONT|B_NOT_CLOSABLE|B_NOT_MOVABLE)
		,fTarget(target)
{
	rect.OffsetTo(B_ORIGIN);
	rect.right -= B_V_SCROLL_BAR_WIDTH;
	fListView = new BListView(rect,"listview",B_SINGLE_SELECTION_LIST
							,B_FOLLOW_ALL,B_WILL_DRAW);
	fListView->SetInvocationMessage(new BMessage(M_LIST_INVOKE));
	BScrollView *scroll = new BScrollView("scroll",fListView,B_FOLLOW_ALL,B_WILL_DRAW,false,true);
	
	AddChild(scroll);
	BStringItem item("Temp");
	fListView->AddItem(&item);
	int32 height=static_cast<int32>( ceil(display*(item.Height()+1)) );
	fListView->RemoveItem(&item);
	ResizeTo(rect.Width(),height);
	fListView->MakeFocus(true);
}

/***********************************************************
 * Destructor
 ***********************************************************/
ComboWindow::~ComboWindow()
{
	fListView->SetInvocationMessage(NULL);
}

/***********************************************************
 * AddItem
 ***********************************************************/
void
ComboWindow::AddItem(const char* text)
{
	fListView->AddItem(new BStringItem(text));
}

/***********************************************************
 * MessageReceived
 ***********************************************************/
void
ComboWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
	case M_LIST_INVOKE:
	{
		int32 sel = fListView->CurrentSelection();
		if(sel<0)
			break;
		BStringItem *item = cast_as(fListView->ItemAt(sel),BStringItem);
		if(!item)
			break;
		BMessage msg(M_LIST_SEL_CHANGED);
		msg.AddString("text",item->Text());
		msg.AddInt32("index",fListView->IndexOf(item));
		fTarget->Looper()->PostMessage(&msg,fTarget);
		break;
	}
	default:
		BWindow::MessageReceived(message);
	}
}

/***********************************************************
 * Select
 ***********************************************************/
void
ComboWindow::Select(int32 index)
{
	if(index<0)
		index=0;
	fListView->Select(index);
	fListView->ScrollToSelection();
}

/***********************************************************
 * QuitRequested
 ***********************************************************/
bool
ComboWindow::QuitRequested()
{
	fTarget->SetWindowNULL();
	return BWindow::QuitRequested();
}