#include "FlyoutWindow.h"
#include "FlyoutBox.h"

#include <TextControl.h>
#include <List.h>
#include <stdlib.h>
#include <string.h>
#include <Bitmap.h>
#include <Picture.h>
#include <PictureButton.h>
#include <Polygon.h>
#include <ClassInfo.h>
#include <Debug.h>
#include <stdio.h>
#include <Screen.h>

#define BUTTON_WIDTH 16

enum{
	M_SHOW_LIST = 'mShL',
	M_TEXT_MODIFIED = 'mTEc',
	M_TEXT_CHANGED = 'mTcD'
};

FlyoutBox::FlyoutBox(BRect rect,const char* name,
			const char *wintitle, uint32 resize, uint32	flags)
//		:BView(rect,name,resize,flags)
		:BBox(rect,name,resize,flags,B_PLAIN_BORDER)
//		,BInvoker(message,NULL)
		,fSubsetWindow(NULL)
		,fDisplayCount(0)
		,fCurrentIndex(-1)
		,fFlagsChanged(false)
{
	rect.OffsetTo(B_ORIGIN);
	rect.right -= BUTTON_WIDTH;

	// Make arrow button
	BPicture *upPict = MakeUpStatePicture();
	BPicture *downPict = MakeDownStatePicture();
	BRect buttonRect(Bounds());

	//buttonRect.OffsetBy(0,1);
	buttonRect.left = buttonRect.right-BUTTON_WIDTH;
	BPictureButton *picBtn = new BPictureButton(buttonRect,
												"_PictureButton",
												upPict,
												downPict,
												new BMessage(M_SHOW_LIST),
												B_TWO_STATE_BUTTON);
	AddChild(picBtn);
	delete upPict;
	delete downPict;
	windowtitle=wintitle;
}

FlyoutBox::~FlyoutBox(void)
{
	if(fSubsetWindow)
		fSubsetWindow->PostMessage(B_QUIT_REQUESTED);
}

void FlyoutBox::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case M_SHOW_LIST:
		{
//			if(fSubsetWindow)
//				HideList();
//			else
				ShowList();
			break;
		}
		default:
			BView::MessageReceived(message);
	}
}

void FlyoutBox::AttachedToWindow(void)
{
	if(Parent())
		SetViewColor(Parent()->ViewColor());
	BPictureButton *button = cast_as(FindView("_PictureButton"),BPictureButton);
	button->SetTarget(this);
	
	fMessageFilter = new FlyoutFilter(this,B_ANY_DELIVERY,B_ANY_SOURCE);
	Window()->AddCommonFilter(fMessageFilter);
}

void FlyoutBox::DetachedFromWindow(void)
{
}

void FlyoutBox::AddDisplayView(BView *view)
{
	if(!view)
		return;
	
	if(displayview)
		debugger("A FlyoutBox may only have one displayview at any time");
	
	if(!windowview)
		ResizeTo(view->Bounds().Width(),Bounds().Height());
	displayview=view;
	AddChild(displayview);
}

BView *FlyoutBox::RemoveDisplayView(void)
{
	RemoveChild(displayview);
	BView *v=displayview;
	displayview=NULL;
	return v;
}

void FlyoutBox::AddWindowView(BView *view)
{
	if(!view)
		return;
	
	if(windowview)
		debugger("A FlyoutBox may only have one windowview at any time");
	
	ResizeTo(view->Bounds().Width(),Bounds().Height());
	windowview=view;
	AddChild(windowview);
}

BView *FlyoutBox::RemoveWindowView(void)
{
	RemoveChild(windowview);
	BView *v=windowview;
	windowview=NULL;
	return v;
}

void FlyoutBox::SetTitle(const char *title)
{
	windowtitle=title;
}

void FlyoutBox::ShowList(void)
{
	if(fSubsetWindow)
		return;
	BRect rect(ConvertToScreen(Bounds()));

	BScreen *screen=new BScreen(Window());
	BRect screenrect(screen->Frame());
	delete screen;
	
	rect.top = rect.bottom;

	if(windowview)
		rect.bottom = rect.top + windowview->Bounds().Height();
	else
		rect.bottom = rect.top + 100;

	fSubsetWindow = new FlyoutWindow(rect,windowtitle.String(),windowview,this);
	rect=fSubsetWindow->Frame();
	
	// Check to make sure the menu will completely show on-screen
	if(!screenrect.Contains(rect))
	{
		// This occurs when the thing isn't quite on-screen
		if(rect.right>screenrect.right)
			fSubsetWindow->MoveBy(screenrect.right-rect.right,0);

		// This occurs when the combobox is at the bottom of the screen, in which case
		// we show the menu above the combobox instead of below
		if(rect.bottom>screenrect.bottom)
			fSubsetWindow->MoveBy(0,-2-rect.Height());
	}
	
	fSubsetWindow->Show();
}

void FlyoutBox::HideList(void)
{
	if(!fSubsetWindow)
		return;
	fSubsetWindow->Hide();
	fSubsetWindow->PostMessage(B_QUIT_REQUESTED);
}

void FlyoutBox::SetWindowNULL(void)
{
	if(Window()->Lock())
	{
		fSubsetWindow = NULL;
		BPictureButton *button = cast_as(FindView("_PictureButton"),BPictureButton);
		button->SetValue(0);	
		Window()->Unlock();
		if(fFlagsChanged)
		{
			Window()->SetFlags(Window()->Flags() & ~B_WILL_ACCEPT_FIRST_CLICK);
			fFlagsChanged=false;
		}
		if(!Window()->IsActive())
			Window()->Activate();
	}
}

void FlyoutBox::SetEnabled(bool enable)
{
	BPictureButton *button = cast_as(FindView("_PictureButton"),BPictureButton);
	button->SetEnabled(enable);
}

void FlyoutBox::KeyDown(const char* bytes,int32 numBytes)
{
/*	if(numBytes > 0 && (bytes[0] == B_UP_ARROW||bytes[0] == B_DOWN_ARROW))
	{
		int32 count = fTextList.CountItems();
		switch(bytes[0])
		{
		case B_UP_ARROW:
			if(fCurrentIndex-1 >= 0)
			{
				fCurrentIndex--;
				const char *text = (const char*)fTextList.ItemAt(fCurrentIndex);
				if(!text)
					break;
				fTextControl->SetText(text);
			}
			break;
		case B_DOWN_ARROW:
			if(fCurrentIndex+1 < count)
			{
				fCurrentIndex++;
				const char *text = (const char*)fTextList.ItemAt(fCurrentIndex);
				if(!text)
					break;
				fTextControl->SetText(text);
			}
			break;
		}
	}else
*/		BView::KeyDown(bytes,numBytes);
}

BPicture* FlyoutBox::MakeUpStatePicture(void)
{
	BRect rect(0,0,BUTTON_WIDTH-1,Bounds().Height()-1);
	BView *view = new BView(rect,"offview",0,0);
	BBitmap *bitmap = new BBitmap(rect,B_CMAP8,true);
	BPicture *pict;
	
	bitmap->AddChild(view);
	bitmap->Lock();				
	view->BeginPicture(new BPicture);
	view->SetHighColor(ui_color(B_PANEL_BACKGROUND_COLOR));
	view->FillRect(rect); 
	view->BeginLineArray(11);
	// Draw Outer Bevel
	view->AddLine(rect.LeftTop(), rect.LeftBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	view->AddLine(rect.LeftTop(), rect.RightTop(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	rect.top++;
	view->AddLine(rect.LeftTop(), rect.RightTop(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_3_TINT));
	
	view->AddLine(rect.LeftBottom(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_MAX_TINT));
	rect.bottom--;
	view->AddLine(rect.LeftBottom(), rect.RightBottom(),
			ui_color(B_PANEL_BACKGROUND_COLOR));
	view->AddLine(rect.RightTop(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_MAX_TINT));
	rect.right--;
	view->AddLine(rect.RightTop(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_2_TINT));
	rect.InsetBy(1,1);
	// Draw Inner Bevel
	view->AddLine(rect.LeftTop(), rect.RightTop(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_MAX_TINT));
	view->AddLine(rect.LeftTop(), rect.LeftBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_MAX_TINT));
	view->AddLine(rect.LeftBottom(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	view->AddLine(rect.RightTop(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	view->EndLineArray();
	
	// Draw arrow
	rect.InsetBy(3,3);
	rect.OffsetBy(0,3);
	
	BPoint points[3];
	points[0] = rect.LeftTop();
	points[1] = rect.RightTop();
	points[2].Set(rect.left+rect.Width()/2,rect.top+8/2-1);
	BPolygon polygon(points,3);
	view->SetHighColor(tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_MAX_TINT));
	view->FillPolygon(&polygon);
	
	pict = view->EndPicture();
	bitmap->Unlock();
	delete bitmap;
	return pict;
}

BPicture* FlyoutBox::MakeDownStatePicture(void)
{
	BRect rect(0,0,BUTTON_WIDTH-1,Bounds().Height()-1);
	BView *view = new BView(rect,"offview",0,0);
	BBitmap *bitmap = new BBitmap(rect,B_CMAP8,true);
	BPicture *pict;
	
	bitmap->AddChild(view);
	bitmap->Lock();				
	view->BeginPicture(new BPicture);
	view->SetHighColor(tint_color(ui_color(B_PANEL_BACKGROUND_COLOR),B_DARKEN_2_TINT));
	view->FillRect(rect); 
	view->BeginLineArray(11);
	// Draw Outer Bevel
	view->AddLine(rect.LeftTop(), rect.LeftBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	view->AddLine(rect.LeftTop(), rect.RightTop(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	rect.top++;
	view->AddLine(rect.LeftTop(), rect.RightTop(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_3_TINT));
	
	view->AddLine(rect.LeftBottom(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_MAX_TINT));
	rect.bottom--;
	view->AddLine(rect.LeftBottom(), rect.RightBottom(),
			ui_color(B_PANEL_BACKGROUND_COLOR));
	view->AddLine(rect.RightTop(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_MAX_TINT));
	rect.right--;
	view->AddLine(rect.RightTop(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_2_TINT));
	// Draw Inner Bevel
	rect.InsetBy(1,1);
	view->AddLine(rect.LeftTop(), rect.RightTop(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	view->AddLine(rect.LeftTop(), rect.LeftBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_1_TINT));
	view->AddLine(rect.LeftBottom(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_1_TINT));
	view->AddLine(rect.RightTop(), rect.RightBottom(),
			tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_LIGHTEN_1_TINT));
	view->EndLineArray();
	
	// Draw arrow
	rect.InsetBy(3,3);
	rect.OffsetBy(0,3);
	BPoint points[3];
	points[0] = rect.LeftTop();
	points[1] = rect.RightTop();
	points[2].Set(rect.left+rect.Width()/2,rect.top+8/2-1);
	BPolygon polygon(points,3);
	view->SetHighColor(tint_color(ui_color(B_PANEL_BACKGROUND_COLOR), B_DARKEN_MAX_TINT));
	view->FillPolygon(&polygon);
	
	pict = view->EndPicture();
	bitmap->Unlock();
	delete bitmap;
	return pict;
}

FlyoutFilter::FlyoutFilter(BView *view,
	message_delivery delivery,message_source source)
: BMessageFilter(delivery,source),
	fView(view)
{

}

FlyoutFilter::~FlyoutFilter(void)
{
}

filter_result FlyoutFilter::Filter(BMessage *message,BHandler **target)
{
	switch(message->what)
	{
		case B_KEY_DOWN:
		{
			const char* bytes;
			message->FindString("bytes",&bytes);
			if(bytes[0] == B_UP_ARROW || bytes[0] == B_DOWN_ARROW)
			{
				((FlyoutBox*)fView)->KeyDown(bytes,1);
				return B_SKIP_MESSAGE;
			}
			break;
		}
		default:
		{
			break;
		}
	}
	return B_DISPATCH_MESSAGE;
}
